from ams_instrument.ams_instrument import ams_instrument
from abc import ABC, abstractmethod

class ams_ppmu_instrument(ams_instrument):
    """
    This interface defines the functionality of an PPMU instrument
    """

    def __init__(self, device_dict: dict) -> object:
        """__init__ 
        Initializes the ams_PPMU_instrument object

        Args:
            device_dict (dict): dictionary of instrument settings

        Returns:
            object: ams_PPMU_instrument object
        """
        pass


    @abstractmethod
    def source_voltage(
        self, 
        voltage_level: float, 
        current_limit: float, 
        channel_name: str,
        voltage_range: float=None,
        current_range: float=None
    ):
        """source_voltage 
        This method sources out voltage to a certain channel with a defined current limit

        Args:
            voltage_level (float): PPMU terminal voltage
            current_limit (float): PPMU current limit
            channel_name (str): Name of the channel on PPMU in use
            voltage_range (float, optional): voltage range - if not given, no new limit is set. Defaults to None.
            current_range (float, optional): current range - if not given, no new limit is set. Defaults to None.
        """
        pass


    @abstractmethod
    def source_current(
        self, 
        current_level: float, 
        voltage_limit: float, 
        channel_name: str,
        voltage_range: float=None,
        current_range: float=None
    ):
        """source_current 
        This method sources out current to a certain channel with a defined voltage limit

        Args:
            current_level (float): PPMU terminal current
            voltage_limit (float): PPMU voltage limit
            channel_name (str): Name of the channel on PPMU in use
            voltage_range (float, optional): voltage range - if not given, no new limit is set. Defaults to None.
            current_range (float, optional): current range - if not given, no new limit is set. Defaults to None.
        """
        pass


    @abstractmethod
    def measure_voltage(
        self, 
        channel_name: str,
        range: str=None, 
    ) -> float:
        """measure_voltage 
        This method measures the voltage on a certain channel with given parameters

        Args:
            channel_name (str): PPMU channel name
            range (str, optional): measurement range. Defaults to None.

        Returns:
            float: measured voltage
        """
        pass


    @abstractmethod
    def measure_current(
        self, 
        channel_name: str,
        range: str=None, 
    ) -> float:
        """
        This method measures the current on a certain channel with given parameters

        Args:
            channel_name (str): PPMU channel name
            range (str, optional): measurement range. Defaults to None.

        Returns:
            float: measured current
        """
        pass


    @abstractmethod
    def stop_sourcing(
        self,
        channel_name: str
    ):
        """stop_sourcing 
        This method stops sourcing on the given terminal

        Args:
            channel_name (str): name of the channel
        """
        pass


    @abstractmethod
    def ppmu_terminal_enable(
        self,
        channel_name: str,
        enable: bool=False
    ):
        """ppmu_terminal_enable 
        This method sets the state of the terminal of PPMU

        Args:
            channel_name (str): name of the PPMU channel
            enable (bool, optional): set this true to activate. Defaults to False.
        """
