from ams_ppmu import ams_ppmu_instrument
import logging, nidigital
from enum import Enum

my_logger = logging.getLogger(__name__)


class ams_PXI_PPMU(ams_ppmu_instrument):
    """
    This interface defines the functionality of an PPMU instrument
    """

    class PPMU_current_range(Enum):
        micro_2 = 2e-6
        micro_32 = 32e-6
        micro_128 = 128e-6
        mili_2 = 2e-3
        mili_32 = 32e-3


    def __init__(self, device_dict: dict) -> object:
        """__init__ 
        Initializes the ams_PPMU_instrument object
        If the instance field is included into the dictionary then the class will use it as instrument reference to the HW session

        Args:
            device_dict (dict): dictionary of instrument settings

        Returns:
            object: ams_PPMU_instrument object
        """
        self.settings_dict = device_dict
        self.pin_map_file = device_dict['Details']['ni_pin_map_file']
        if 'instance' not in self.settings_dict['Details'].keys():
            my_logger.debug('Initialize new session')
            self.handler = nidigital.Session(
                resource_name=self.settings_dict['Details']['max_alias'],
                reset_device=False
            )
        else:
            my_logger.debug('Session is injected')
            self.handler = self.settings_dict['Details']['instance']


    def source_voltage(
        self, 
        voltage_level: float, 
        current_limit: float, 
        channel_name: str,
        voltage_range: float=None,
        current_range: float=None
    ):
        """source_voltage 
        This method sources out voltage to a certain channel with a defined current limit

        Args:
            voltage_level (float): PPMU terminal voltage
            current_limit (float): PPMU current limit
            channel_name (str): Name of the channel on PPMU in use
            voltage_range (float, optional): voltage range - if not given, no new limit is set. Defaults to None.
            current_range (float, optional): current range - if not given, no new limit is set. Defaults to None.
        """
        self.handler.channels[channel_name].ppmu_output_function = nidigital.PPMUOutputFunction.VOLTAGE
        self.handler.channels[channel_name].ppmu_voltage_level = voltage_level
        self.handler.channels[channel_name].ppmu_current_limit_range = current_limit
        if current_range != None:
            self.handler.channels[channel_name].ppmu_current_level_range = current_range
        self.handler.channels[channel_name].ppmu_source()


    def source_current(
        self, 
        current_level: float, 
        voltage_limit: float, 
        channel_name: str,
        voltage_range: float=None,
        current_range: float=None
    ):
        """source_current 
        This method sources out current to a certain channel with a defined voltage limit

        Args:
            current_level (float): PPMU terminal current
            voltage_limit (float): PPMU voltage limit
            channel_name (str): Name of the channel on PPMU in use
            voltage_range (float, optional): voltage range - if not given, no new limit is set. Defaults to None.
            current_range (float, optional): current range - if not given, no new limit is set. Defaults to None.
        """
        self.handler.channels[channel_name].ppmu_output_function = nidigital.PPMUOutputFunction.CURRENT
        self.handler.channels[channel_name].ppmu_current_level = current_level
        self.handler.channels[channel_name].ppmu_voltage_limit_high = voltage_limit
        if current_range != None:
            self.handler.channels[channel_name].ppmu_current_level_range = current_range
        self.handler.channels[channel_name].ppmu_source()


    def measure_voltage(
        self, 
        channel_name: str,
        range: str=None, 
    ) -> float:
        """measure_voltage 
        This method measures the voltage on a certain channel with given parameters

        Args:
            channel_name (str): PPMU channel name
            range (str, optional): measurement range. Defaults to None.

        Returns:
            float: measured voltage
        """
        measurement = self.handler.channels[channel_name].ppmu_measure(nidigital.PPMUMeasurementType.VOLTAGE)
        my_logger.debug(measurement)
        return measurement[0]


    def measure_current(
        self, 
        channel_name: str,
        range: str=None, 
    ) -> float:
        """
        This method measures the current on a certain channel with given parameters

        Args:
            channel_name (str): PPMU channel name
            range (str, optional): measurement range. Defaults to None.

        Returns:
            float: measured current
        """
        measurement = self.handler.channels[channel_name].ppmu_measure(nidigital.PPMUMeasurementType.CURRENT)
        my_logger.debug(measurement)
        return measurement[0]


    def stop_sourcing(
        self,
        channel_name: str
    ):
        """stop_sourcing 
        This method stops sourcing on the given terminal

        Args:
            channel_name (str): name of the channel
        """
        if self.handler.channels[channel_name].ppmu_output_function == nidigital.PPMUOutputFunction.VOLTAGE:
            self.handler.channels[channel_name].ppmu_voltage_level = 0
        else:
            self.handler.channels[channel_name].ppmu_current_level = 0
        self.handler.channels[channel_name].ppmu_source()


    def GetInstrumentError(self) -> str:
        """IdentifyInstrument 
        Gets the identifier of the instrument as string

        Returns:
            ID_string [String]: The identification string from the instrument
        """
        return self.handler._get_error()


    def IdentifyInstrument(self) -> str:
        """GetInstrumentError
        This function returns the error string from the instrument

        Returns:
            Error_string [String]: The error string from the instrument
        """
        return self.handler.logical_name


    def ResetInstrumehnt(self) -> str:
        """ResetInstrumehnt 
        This function resets the instrument
        """
        return self.handler.reset()


    def ppmu_terminal_enable(
        self,
        channel_name: str,
        enable: bool=False
    ):
        """ppmu_terminal_enable 
        This method sets the state of the terminal of PPMU

        Args:
            channel_name (str): name of the PPMU channel
            enable (bool, optional): set this true to activate. Defaults to False.
        """
        if enable:
            self.handler.channels[channel_name].selected_function(nidigital.SelectedFunction.PPMU)
        else:
            self.handler.channels[channel_name].selected_function(nidigital.SelectedFunction.DISCONNECT)
        